<?php /*

**************************************************************************

Plugin Name:  David's Ultra Quicktags
Plugin URI:   http://www.davidgagne.net/tags/plugins/
Description:  Adds new buttons to the Editor window (Amazon Links, Amazon Search, etc.).
Version:      1.0.33
Author:       David Vincent Gagne
Author URI:   http://www.davidgagne.net/

**************************************************************************

Copyright (C) 2009 David Vincent Gagne

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

/*-----------------------------------------------------
Class declaration
-----------------------------------------------------*/
class DavidsUltraQuicktags {

/*-----------------------------------------------------
	class variables
-----------------------------------------------------*/
	// The $version is the version of this plugin.  This is used so users know if they have the most
	// recent version and so that the WordPress plugins system can alert a user if a new version
	// is available
	var $version = '1.0.33';
	// The $settings array stores ...
	var $settings = array();
	// The $defaultsettings array stores the settings that the plugin will use if the user does not
	// specify his or her own options.
	var $defaultsettings = array();
	// The $adminwarned variable is used so that the plugin knows whether ...
	var $adminwarned = FALSE;
	var $plugin_version = "1.0.9";
	var $plugin_dir = "../wp-content/plugins/davids-ultra-quicktags/";
	var $resources_dir = "../wp-content/plugins/davids-ultra-quicktags/resources/";

/*-----------------------------------------------------
	class initialization
-----------------------------------------------------*/
	function DavidsUltraQuicktags() {
		// The $wp_db_version stores the WordPress database version.
		// The $wpmu_version stores the WordPress MultiUser installation version (for cases
		// where the user is running WordPress on a shared host).
		global $wp_db_version, $wpmu_version;

/*-----------------------------------------------------
		This version requires WordPress 2.6+
-----------------------------------------------------*/
		if ( !function_exists('plugins_url') ) {
			if ( isset( $_GET['activate'] ) ) {
				wp_redirect( 'plugins.php?deactivate=true' );
				exit();
			} else {
/*-----------------------------------------------------
				Replicate deactivate_plugins()
-----------------------------------------------------*/
				$current = get_option('active_plugins');
				$plugins = array( 'davids-ultra-quicktags/davids-ultra-quicktags.php', 'davids-ultra-quicktags.php' );
				foreach ( $plugins as $plugin ) {
					if( !in_array( $plugin, $current ) ) continue;
					array_splice( $current, array_search( $plugin, $current ), 1 );
				}
				update_option('active_plugins', $current);
				add_action( 'admin_notices', array(&$this, 'WPVersionTooOld') );
				return;
			}
		}

/*-----------------------------------------------------
		Create default settings array
			button = whether to display the button by default on the editor screen
			associateid = the Amazon associateid you want to credit with the link
			targetnew = whether to target links in a new window / tab by default
-----------------------------------------------------*/
		$this->defaultsettings = apply_filters( 'dvg_defaultsettings', array(
			'amazonlink' => array(
				'button'		=> 1,
				'associateid'	=> 'davidgagnenet-20',
				'targetnew'	=> 0
			),
			'amazonsearch' => array(
				'button'		=> 1,
				'associateid'	=> 'davidgagnenet-20',
				'targetnew'	=> 0
			),
			'dvgspan' => array(
				'button'		=> 1,
				'class'		=> 'dropcap',
				'targetnew'	=> 0
			),
			'dvgdiv' => array(
				'button'		=> 1,
				'class'		=> 'pullquote',
				'targetnew'	=> 0
			),
			'tinymceline'		=> 3
		) );

/*-----------------------------------------------------
		Setup the settings by using the default as a base and then adding in any changed values
		This allows settings arrays from old versions to be used even though they are missing values
-----------------------------------------------------*/
		$usersettings = (array) get_option('dvg_options');
		$this->settings = $this->defaultsettings;
		if ( $usersettings !== $this->defaultsettings ) {
			foreach ( (array) $usersettings as $key1 => $value1 ) {
				if ( is_array($value1) ) {
					foreach ( $value1 as $key2 => $value2 ) {
						$this->settings[$key1][$key2] = $value2;
					}
				} else {
					$this->settings[$key1] = $value1;
				}
			}
		}

/*-----------------------------------------------------
		Upgrade settings
-----------------------------------------------------*/
		if ( empty($this->settings['version']) ) {
			$usersettings = (array) get_option('dvg_options');
			foreach ( $this->defaultsettings as $type => $setting ) {
				if ( !is_array($this->defaultsettings[$type]) ) continue;
				if ( isset($usersettings[$type]['button']) )
					unset($usersettings[$type]['button']); // Reset buttons
			}
			$usersettings['version'] = $this->version;
			update_option( 'dvg_options', $usersettings );
		}

/*-----------------------------------------------------
		Register general hooks
-----------------------------------------------------*/
		add_action( 'admin_menu', array(&$this, 'RegisterSettingsPage') );
		add_filter( 'plugin_action_links', array(&$this, 'AddPluginActionLink'), 10, 2 );
		add_action( 'admin_post_dvgsettings', array(&$this, 'POSTHandler') );
		add_action( 'admin_head', array(&$this, 'Head') );
		if ( $wp_db_version < 8989 && 'update.php' == basename( $_SERVER['PHP_SELF'] ) && 'upgrade-plugin' == $_GET['action'] && FALSE !== strstr( $_GET['plugin'], 'davids-ultra-quicktags' ) )
			add_action( 'admin_notices', array(&$this, 'AutomaticUpgradeNotice') );
/*-----------------------------------------------------
		Register editor button hooks
-----------------------------------------------------*/
		add_filter( 'tiny_mce_version', array(&$this, 'tiny_mce_version') );
		add_filter( 'mce_external_plugins', array(&$this, 'mce_external_plugins') );
		add_action( 'edit_form_advanced', array(&$this, 'AddQuicktagsAndFunctions') );
		add_action( 'edit_page_form', array(&$this, 'AddQuicktagsAndFunctions') );
		if ( 1 == $this->settings['tinymceline'] )
			add_filter( 'mce_buttons', array(&$this, 'mce_buttons') );
		else
			add_filter( 'mce_buttons_' . $this->settings['tinymceline'], array(&$this, 'mce_buttons') );

		if ( is_admin() ) {
			if ( in_array( basename($_SERVER['PHP_SELF']), apply_filters( 'dvg_editor_pages', array('post-new.php', 'page-new.php', 'post.php', 'page.php') ) ) ) {
				add_action( 'admin_head', array(&$this, 'EditorCSS') );
			}
		}

/*-----------------------------------------------------
		Hide the donate button on WPMU installs as admins probably don't want it there
-----------------------------------------------------*/
		if ( !empty($wpmu_version) ) add_filter( 'dvg_donatebutton', array(&$this, 'ReturnFalse'), 5 );
	}

/*-----------------------------------------------------
	This function gets called when the minimum WordPress version isn't met
-----------------------------------------------------*/
	function WPVersionTooOld() {
		echo '<div class="error"><p>' . sprintf( __( 'This version of <strong>David\'s Ultra Quicktags</strong> requires WordPress 2.6 or newer. Please <a href="%1$s">upgrade</a>.', 'davids-ultra-quicktags' ), 'http://codex.wordpress.org/Upgrading_WordPress' ) . "</p></div>\n";
	}

/*-----------------------------------------------------
	Register the settings page that allows plugin configuration
-----------------------------------------------------*/
	function RegisterSettingsPage() {
		add_options_page( __("David's Ultra Quicktags Configuration", 'davids-ultra-quicktags'), __('Ultra Quicktags', 'davids-ultra-quicktags'), 'manage_options', 'davids-ultra-quicktags', array(&$this, 'SettingsPage') );
	}

/*-----------------------------------------------------
	Add a link to the settings page to the plugins list
-----------------------------------------------------*/
	function AddPluginActionLink( $links, $file ) {
		static $this_plugin;
		if( empty($this_plugin) ) $this_plugin = plugin_basename(__FILE__);
		if ( $file == $this_plugin ) {
			$settings_link = '<a href="' . admin_url( 'options-general.php?page=davids-ultra-quicktags' ) . '">' . __('Settings', 'davids-ultra-quicktags') . '</a>';
			array_unshift( $links, $settings_link );
		}
		return $links;
	}

/*-----------------------------------------------------
	Break the browser cache of TinyMCE
-----------------------------------------------------*/
	function tiny_mce_version( $version ) {
		return $version . '-dvg' . $this->version . 'line' . $this->settings['tinymceline'];
	}

/*-----------------------------------------------------
	Load the custom TinyMCE plugin
-----------------------------------------------------*/
	function mce_external_plugins( $plugins ) {
		$plugins['davidsultraquicktags'] = plugins_url('/davids-ultra-quicktags/resources/tinymce3/editor_plugin.js');
		return $plugins;
	}

/*-----------------------------------------------------
	Add the custom TinyMCE buttons
-----------------------------------------------------*/
	function mce_buttons( $buttons ) {
		array_push( $buttons, 'dvgamazonlink', 'dvgamazonsearch', 'dvgspan', 'dvgdiv' );
		return $buttons;
	}

	// Hide TinyMCE buttons the user doesn't want to see + some misc editor CSS
	function EditorCSS() {
		echo "<style type='text/css'>\n";
		$buttons2hide = array();
		if ( 1 != $this->settings['amazonlink']['button'] )     $buttons2hide[] = 'amazonlink';
		if ( 1 != $this->settings['amazonsearch']['button'] )   $buttons2hide[] = 'amazonsearch';
		if ( 1 != $this->settings['dvgspan']['button'] )   $buttons2hide[] = 'dvgspan';
		if ( 1 != $this->settings['dvgdiv']['button'] )   $buttons2hide[] = 'dvgdiv';
		echo '	.mce_dvg' . implode( ', .mce_dvg', $buttons2hide ) . " { display: none !important; }\n";
		echo "</style>\n";
	}
	
/*-----------------------------------------------------
	Add the old style buttons to the non-TinyMCE editor views and output all of the JS for the button function + dialog box
-----------------------------------------------------*/
	function AddQuicktagsAndFunctions() {
		$types = array(
			'amazonlink'     => array(
				__('Amazon Link', 'davids-ultra-quicktags'),
				__('Link directly to a product by its ASIN or ISBN.', 'davids-ultra-quicktags'),
				__('amazonlink', 'davids-ultra-quicktags')
			),
			'amazonsearch' => array(
				__('Amazon Search', 'davids-ultra-quicktags'),
				__('Enter a term to use for an Amazon search.', 'davids-ultra-quicktags'),
				__('amazonsearch', 'davids-ultra-quicktags')
			),
			'dvgspan' => array(
				__('span', 'davids-ultra-quicktags'),
				__('Add a custom span tag.', 'davids-ultra-quicktags'),
				__('dvgspan', 'davids-ultra-quicktags')
			),
			'dvgdiv' => array(
				__('div', 'davids-ultra-quicktags'),
				__('Add a custom div tag.', 'davids-ultra-quicktags'),
				__('dvgdiv', 'davids-ultra-quicktags')
			)
		);

		$buttonhtml = $datajs = '';
		foreach ( $types as $type => $strings ) {
/*-----------------------------------------------------
			HTML for quicktag button
-----------------------------------------------------*/
			if ( 1 == $this->settings[$type]['button'] )
				$buttonshtml .= '<input type="button" class="ed_button" onclick="DVGButtonClick(\'' . $type . '\')" title="' . $strings[1] . '" value="' . $strings[0] . '" id="' . $type . '" />';

		}

		?>
<script type="text/javascript">
// <![CDATA[

// This function is run when a button is clicked.
function DVGButtonClick( tag ) {
	// This stuff is just for debugging.
	//	alert( tag );
	//	alert( document.getElementById(tag).id );
	var dvgi = edButtons.length;

	switch(tag){
		case 'amazonlink':
			dvgi = dvgi - 4;
			var title = prompt('Title Tag', 'Buy @ Amazon');
			var ASIN = prompt('Enter the ASIN', 'ASIN');
			var alassociateid = '<? echo $this->settings['amazonlink']['associateid']; ?>';
			<?
			if ($this->settings['amazonlink']['targetnew'] == 1){
				echo "var targetnew = ' target=\"_blank\" ';\n";
			}
			else{
				echo "var targetnew = ' ';\n";
			}
			?>
			if (ASIN) {
				newtext = '<a title="' + title + '"' + targetnew + 'href="http://www.amazon.com/exec/obidos/ASIN/' + ASIN + '/' + alassociateid + '">';
			}
			break;
		case 'amazonsearch':
			dvgi = dvgi - 3;
			var title = prompt('Title Tag', 'Search @ Amazon');
			var SearchTerm = prompt('Enter the search term', 'Foo Fighters');
			var asassociateid = '<? echo $this->settings['amazonsearch']['associateid']; ?>';
			<?
			if ($this->settings['amazonsearch']['targetnew'] == 1){
				echo "var targetnew = ' target=\"_blank\" ';\n";
			}
			else{
				echo "var targetnew = ' ';\n";
			}
			?>
			if (SearchTerm) {
				newtext = '<a title="' + title + '"' + targetnew + 'href="http://www.amazon.com/exec/obidos/external-search/002-0187043-4860819?index=blended&amp;tag=' + asassociateid + '&amp;keyword=' + SearchTerm + '">';
			}
			break;
		case 'dvgspan':
			dvgi = dvgi - 2;
			var spanclass = prompt('Enter the span class', '<? echo $this->settings['dvgspan']['associateid']; ?>');
			if (spanclass) {
				newtext = '<span class="' + spanclass + '">';
			}
			break;
		case 'dvgdiv':
			dvgi = dvgi - 1;
			var divclass = prompt('Enter the div class', '<? echo $this->settings['dvgdiv']['associateid']; ?>');
			if (divclass) {
				newtext = '<div class="' + divclass + '">';
			}
			break;
	}

	edButtons[dvgi].tagStart = newtext;
		
	if ( typeof tinyMCE != 'undefined' && ( ed = tinyMCE.activeEditor ) && !ed.isHidden() ) {
		ed.focus();
		if (tinymce.isIE)
			ed.selection.moveToBookmark(tinymce.EditorManager.activeEditor.windowManager.bookmark);
		var toreplace = tinyMCE.activeEditor.selection.getContent();
		ed.execCommand('mceInsertContent', false, newtext + toreplace + '</span>');
	} else
		edInsertTag(edCanvas, dvgi);
}

// On page load...
jQuery(document).ready(function(){
	// Add the buttons to the HTML view
	jQuery("#ed_toolbar").append('<?php echo $this->js_escape( $buttonshtml ); ?>');
	edButtons[edButtons.length] = new edButton('amazonlink', 'z', '', '</a>' ,'z');
	edButtons[edButtons.length] = new edButton('amazonsearch', 'x', '', '</a>' ,'x');
	edButtons[edButtons.length] = new edButton('dvgspan', 's', '', '</span>' ,'s');
	edButtons[edButtons.length] = new edButton('dvgdiv', 'd', '', '</div>' ,'s');
});
// ]]>
</script>
<?php
	}

/*-----------------------------------------------------
	Handle the submits from the settings page
-----------------------------------------------------*/
	function POSTHandler() {
		global $wpmu_version;

/*-----------------------------------------------------
		permissions check
-----------------------------------------------------*/
		if ( !current_user_can('manage_options') )
			wp_die( __('Cheatin&#8217; uh?') );

/*-----------------------------------------------------
		Form nonce check
-----------------------------------------------------*/
		check_admin_referer('davids-ultra-quicktags');

		$usersettings = (array) get_option('dvg_options');
		$defaults = FALSE;

		switch ( $_POST['dvg-tab'] ) {
			case 'general':
				$fields = array( 'button', 'associateid', 'targetnew' );

/*-----------------------------------------------------
				Check for the defaults button, clear out all values on the page if pressed (which makes the defaults be used)
-----------------------------------------------------*/
				if ( !empty($_POST['dvg-defaults']) ) {
					foreach ( $this->defaultsettings as $type => $settings ) {
						if ( !is_array($this->defaultsettings[$type]) ) continue;
						foreach ( $fields as $setting ) {
							if ( isset($usersettings[$type][$setting]) )
								unset( $usersettings[$type][$setting] );
						}
					}

					$defaults = TRUE;
					break;
				}

/*-----------------------------------------------------
				Copy in the results of the form
-----------------------------------------------------*/
				foreach ( $this->defaultsettings as $type => $settings ) {
					if ( !is_array($this->defaultsettings[$type]) ) continue;
					foreach ( $fields as $setting ) {
						if ( isset($_POST['dvg'][$type][$setting]) )
							$usersettings[$type][$setting] = $_POST['dvg'][$type][$setting];
						else
							$usersettings[$type][$setting] = 0;

/*-----------------------------------------------------
						AssociateID is required, clear if 0
-----------------------------------------------------*/
						if ( 0 === $usersettings[$type][$setting] && in_array( $setting, array( 'associateid' ) ) )
							unset( $usersettings[$type][$setting] );
					}
				}
				break;
		}

		update_option( 'dvg_options', $usersettings );

/*-----------------------------------------------------
		Redirect back to where we came from
-----------------------------------------------------*/
		$redirectto = wp_get_referer();
		$redirectto = ( TRUE == $defaults ) ? add_query_arg( 'defaults', 'true', $redirectto ) : add_query_arg( 'updated', 'true', $redirectto );
		wp_safe_redirect( $redirectto );
	}

/*-----------------------------------------------------
	Output the settings page
-----------------------------------------------------*/
	function SettingsPage() {
		global $wpmu_version;

		$tab = ( !empty($_GET['tab']) ) ? $_GET['tab'] : 'general';

		if ( !empty($_GET['defaults']) ) : ?>
<div id="message" class="updated fade"><p><strong><?php _e('Settings for this tab reset to defaults.', 'davids-ultra-quicktags'); ?></strong></p></div>
<?php endif; ?>

<div class="wrap">

	<form action="https://www.paypal.com/cgi-bin/webscr" method="post">
	<input type="hidden" name="cmd" value="_donations" />
	<input type="hidden" name="business" value="cash@davidgagne.net" />
	<input type="hidden" name="item_name" value="<?php echo attribute_escape( __("David's Ultra Quicktags", 'davids-ultra-quicktags') ); ?>" />
	<input type="hidden" name="no_shipping" value="1" />
	<input type="hidden" name="return" value="http://www.davidgagne.net/about/" />
	<input type="hidden" name="cancel_return" value="http://www.davidgagne.net/tags/plugins/" />
	<input type="hidden" name="cn" value="<?php echo attribute_escape( __('Optional Comment', 'davids-ultra-quicktags') ); ?>" />
	<input type="hidden" name="currency_code" value="USD" />
	<input type="hidden" name="tax" value="0" />
	<input type="hidden" name="lc" value="US" />
	<input type="hidden" name="bn" value="PP-DonationsBF" />

<?php if ( function_exists('screen_icon') ) screen_icon(); ?>
	<h2>
<?php

	_e("David's Ultra Quicktags " . $this->version, 'davids-ultra-quicktags');

	// Want to get rid of the donate button? Alright, if you insist. :(
	// Use the "dvg_donatebutton" filter to return FALSE and it'll go away (much better than editing this file).
	if ( TRUE === apply_filters( 'dvg_donatebutton', TRUE ) )
		echo '		<input type="image" src="https://www.paypal.com/en_US/i/btn/x-click-but04.gif" name="submit" alt="Donate" title="' . attribute_escape( __('Donate to David Vincent Gagne for this plugin via PayPal', 'davids-ultra-quicktags') ) . '" style="vertical-align:middle;" />' . "\n";

?>
	</h2>

	</form>

	<ul class="subsubsub">
<?php
		$tabs = array(
			'credits'     => __('Credits', 'davids-ultra-quicktags')
		);
		$tabhtml = array();

		// If someone wants to remove a tab (for example on a WPMU intall)
		$tabs = apply_filters( 'dvg_tabs', $tabs );

		$class = ( 'general' == $tab ) ? ' class="current"' : '';
		$tabhtml[] = '		<li><a href="' . admin_url( 'options-general.php?page=davids-ultra-quicktags' ) . '"' . $class . '>' . __('General', 'davids-ultra-quicktags') . '</a>';

		foreach ( $tabs as $stub => $title ) {
			$class = ( $stub == $tab ) ? ' class="current"' : '';
			$tabhtml[] = '		<li><a href="' . admin_url( 'options-general.php?page=davids-ultra-quicktags&amp;tab=' . $stub ) . '"' . $class . ">$title</a>";
		}

		echo implode( " |</li>\n", $tabhtml ) . '</li>';
?>

	</ul>

	<form id="dvgsettingsform" method="post" action="admin-post.php" style="clear:both">

	<?php wp_nonce_field('davids-ultra-quicktags'); ?>

	<input type="hidden" name="action" value="dvgsettings" />

<?php


	// Figure out which tab to output
	switch ( $tab ) {
		case 'credits': ?>
	<p><?php _e('This plugin is based on <a href="http://www.viper007bond.com/wordpress-plugins/vipers-video-quicktags/">Viper\'s Video Quicktags</a>, an awesome way to embed videos in your blog.</p><p>Here are some other resources I used to create it, and some people who deserve my thanks:', 'davids-ultra-quicktags'); ?></p>

	<ul>
		<li><?php printf( __('<a href="%s">Kelly Bakst</a>', 'davids-ultra-quicktags'), 'http://kellybakst.com/' ); ?></li>
		<li><?php printf( __('<a href="%s">Granades.com</a>', 'davids-ultra-quicktags'), 'http://granades.com/2007/02/14/adding-quicktags-to-wordpress/' ); ?></li>
		<li><?php printf( __('<a href="%s">Brandissimo.com</a>', 'davids-ultra-quicktags'), 'http://www.brandissimo.com/' ); ?></li>
		<li><?php printf( __('<a href="%s">TinyMCE API Documentation</a>', 'davids-ultra-quicktags'), 'http://wiki.moxiecode.com/index.php/TinyMCE:API' ); ?></li>
	</ul>

<?php
			break; // End credits

		default;
?>
	<p><?php _e('Click the above links to switch between tabs.', 'davids-ultra-quicktags'); ?></p>

	<input type="hidden" name="dvg-tab" value="general" />

	<table class="widefat" style="text-align:center">
		<thead>
			<tr>
				<th scope="col" style="text-align:left"><?php _e('QuickTag', 'davids-ultra-quicktags'); ?></th>
				<th scope="col" style="text-align:center"><?php _e('Show Editor Button?', 'davids-ultra-quicktags'); ?></th>
				<th scope="col" style="text-align:center"><?php _e('Associate ID', 'davids-ultra-quicktags'); ?></th>
				<th scope="col" style="text-align:center"><?php _e('Target New Window?', 'davids-ultra-quicktags'); ?></th>
			</tr>
		</thead>
		<tbody>
			<tr class="alternate">
				<td style="text-align:left">Amazon Product Link</td>
				<td><input name="dvg[amazonlink][button]" type="checkbox" value="1"<?php checked($this->settings['amazonlink']['button'], 1); ?> /></td>
				<td>
					<input name="dvg[amazonlink][associateid]" type="text" value="<?php echo $this->settings['amazonlink']['associateid']; ?>" />
					<input type="hidden" value="<?php echo $this->defaultsettings['amazonlink']['associateid']; ?>" />
				</td>
				<td><input name="dvg[amazonlink][targetnew]" type="checkbox" value="1"<?php checked($this->settings['amazonlink']['targetnew'], 1); ?> /></td>
			</tr>
			<tr>
				<td style="text-align:left">Amazon Search Link</td>
				<td><input name="dvg[amazonsearch][button]" type="checkbox" value="1"<?php checked($this->settings['amazonsearch']['button'], 1); ?> /></td>
				<td>
					<input name="dvg[amazonsearch][associateid]" type="text" value="<?php echo $this->settings['amazonsearch']['associateid']; ?>" />
					<input type="hidden" value="<?php echo $this->defaultsettings['amazonsearch']['associateid']; ?>" />
				</td>
				<td><input name="dvg[amazonsearch][targetnew]" type="checkbox" value="1"<?php checked($this->settings['amazonsearch']['targetnew'], 1); ?> /></td>
			</tr>
		</tbody>
		<thead>
			<tr>
				<th scope="col" style="text-align:left"><?php _e('QuickTag', 'davids-ultra-quicktags'); ?></th>
				<th scope="col" style="text-align:center"><?php _e('Show Editor Button?', 'davids-ultra-quicktags'); ?></th>
				<th scope="col" style="text-align:center"><?php _e('Default Class', 'davids-ultra-quicktags'); ?></th>
				<th scope="col" style="text-align:center">&nbsp;</th>
			</tr>
		</thead>
		<tbody>
			<tr class="alternate">
				<td style="text-align:left">Custom Span</td>
				<td><input name="dvg[dvgspan][button]" type="checkbox" value="1"<?php checked($this->settings['dvgspan']['button'], 1); ?> /></td>
				<td>
					<input name="dvg[dvgspan][associateid]" type="text" value="<?php echo $this->settings['dvgspan']['associateid']; ?>" />
					<input type="hidden" value="<?php echo $this->defaultsettings['dvgspan']['associateid']; ?>" />
				</td>
				<td><input name="dvg[dvgspan][targetnew]" type="hidden" value="0" />&nbsp;</td>
			</tr>
			<tr class="alternate">
				<td style="text-align:left">Custom Div</td>
				<td><input name="dvg[dvgdiv][button]" type="checkbox" value="1"<?php checked($this->settings['dvgdiv']['button'], 1); ?> /></td>
				<td>
					<input name="dvg[dvgdiv][associateid]" type="text" value="<?php echo $this->settings['dvgdiv']['associateid']; ?>" />
					<input type="hidden" value="<?php echo $this->defaultsettings['dvgdiv']['associateid']; ?>" />
				</td>
				<td><input name="dvg[dvgdiv][targetnew]" type="hidden" value="0" />&nbsp;</td>
			</tr>
		</tbody>
	</table>
<?php
			// End General tab
	}
?>

<?php if ( 'credits' != $tab ) : ?>
	<p class="submit">
		<input type="submit" name="dvg-submit" value="<?php _e('Save Changes', 'davids-ultra-quicktags'); ?>" />
		<input type="submit" name="dvg-defaults" id="dvg-defaults" value="<?php _e('Reset Tab To Defaults', 'davids-ultra-quicktags'); ?>" />
	</p>
<?php
				echo "
					<div style=\"float:left;background:#ddd;border:2px outset #b29e90;margin-top:20px;\">
					<div style=\"color:#fff;background:#2a5b8e;margin:0;padding:4px;font:normal medium Futura, Tahoma, verdana, sans-serif;\">Thanks for using my plugin!</div>
					<div style=\"padding:10px 20px 0 20px;\">
					<p>If you like this plugin you should <a href=\"http://wordpress.org/extend/plugins/profile/dvg\">try some of my others</a>.</p>
					<p>You can also <a href=\"http://twitter.com/davidgagne\">follow me on Twitter</a>, <a href=\"http://www.davidgagne.net/\">visit my website</a>, and <a href=\"http://feeds.feedburner.com/davidgagne\">subscribe to my feed via RSS</a>.</p>
					<p style=\"text-align:right;\">Thanks!</p>
					<p style=\"text-align:right;\">dvg</p><hr />
					<ul>
					<li>Plugin Version: {$this->plugin_version}</li>
					<li>Plugin Directory: {$this->plugin_dir}</li>
					<li>Plugin Resources Directory: {$this->resources_dir}</li>
					</ul>
					<br style=\"clear:both;\" />
					</div>
					</div>
				";

 endif; ?>

	</form>
</div>

<?php
		/*
		echo '<pre>';
		print_r( get_option('dvg_options') );
		echo '</pre>';
		*/
	}


/*-----------------------------------------------------
	Output the head stuff
-----------------------------------------------------*/
	function Head() {
		echo "\n<!-- David's Ultra Quicktags v. " . $this->version . " - http://www.davidgagne.net/tags/plugins/ -->\n";
	}

/*-----------------------------------------------------
	Conditionally output debug error text
-----------------------------------------------------*/
	function error( $error ) {
		global $post;

		// If the user can't edit this post, then just silently fail
		if ( empty($post->ID) || ( 'post' == $post->post_type && !current_user_can( 'edit_post', $post->ID ) ) || ( 'page' == $post->post_type && !current_user_can( 'edit_page', $post->ID ) ) )
			return '';

		// But if this user is an admin, then display some helpful text
		return '<em>[' . sprintf( __('<strong>ERROR:</strong> %s', 'davids-ultra-quicktags'), $error ) . ']</em>';
	}

/*-----------------------------------------------------
	Reverse the parts we care about (and probably some we don't) of wptexturize() which gets applied before shortcodes
-----------------------------------------------------*/
	function wpuntexturize( $text ) {
		$find = array( '&#8211;', '&#8212;', '&#215;', '&#8230;', '&#8220;', '&#8217;s', '&#8221;', '&#038;' );
		$replace = array( '--', '---', 'x', '...', '``', '\'s', '\'\'', '&' );
		return str_replace( $find, $replace, $text );
	}

/*-----------------------------------------------------
	WordPress' js_escape() won't allow <, >, or " -- instead it converts it to an HTML entity. This is a "fixed" function that's used when needed.
-----------------------------------------------------*/
	function js_escape($text) {
		$safe_text = addslashes($text);
		$safe_text = preg_replace('/&#(x)?0*(?(1)27|39);?/i', "'", stripslashes($safe_text));
		$safe_text = preg_replace("/\r?\n/", "\\n", addslashes($safe_text));
		$safe_text = str_replace('\\\n', '\n', $safe_text);
		return apply_filters('js_escape', $safe_text, $text);
	}


/*-----------------------------------------------------
	This function always return FALSE (who woulda guessed?)
-----------------------------------------------------*/
	function ReturnFalse() { return FALSE; }
}

/*-----------------------------------------------------
Start this plugin once all other plugins are fully loaded
-----------------------------------------------------*/
add_action( 'plugins_loaded', 'DavidsUltraQuicktags' );

function DavidsUltraQuicktags(){
	global $DavidsUltraQuicktags;
	$DavidsUltraQuicktags = new DavidsUltraQuicktags();
}

?>